/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5;

import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.ListableBeanFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * This class provide a way to interface with spring.
 * Use
 * <pre>
 * &lt;bean id="XXX"
 * 	class="org.boretti.drools.integration.drools5.DroolsBeanFactory"&gt;
 *		&lt;property name="droolsInterface" value="interface name"/&gt;
 * &lt;/bean&gt;
 * </pre>
 * To use it. The passed interface must have been annotated with the
 * right annotation.
 * @author mbo
 *
 */
public class DroolsBeanFactory implements FactoryBean,BeanFactoryAware {
	
	private final Logger logger = Logger.getLogger(DroolsBeanFactory.class);

	private Class<?> droolsInterface;
	
	private BeanFactory beanFactory;
	
	private DroolsProvider droolsProvider = new DroolsProvider();
	
	@Override
	public Object getObject() throws Exception {
		Object o = droolsProvider.getService(droolsInterface);
		for(Method m:droolsInterface.getMethods()) {
			Autowired a = null;
			try {
				a=m.getAnnotation(Autowired.class);
			} catch (NullPointerException ex) {}
			if (a!=null) {
				String methodName = m.getName();
				if (methodName.startsWith("set") && m.getParameterTypes().length==1) {
					String fieldName = methodName.substring(3,4).toLowerCase()+methodName.substring(4); 
					if (beanFactory instanceof ListableBeanFactory) {
						ListableBeanFactory lbf = (ListableBeanFactory)beanFactory;
						String names[] = lbf.getBeanNamesForType(m.getParameterTypes()[0]);
						if (names.length>0) {
							if (logger.isDebugEnabled()) logger.debug("Injection into "+fieldName);
							m.invoke(o, lbf.getBean(names[0]));
						}
					}
				}
			}
		}
		return o;
	}

	@Override
	public Class<?> getObjectType() {
		return droolsInterface;
	}

	@Override
	public boolean isSingleton() {
		return false;
	}

	public void setDroolsInterface(Class<?> droolsInterface) {
		this.droolsInterface = droolsInterface;
	}

	@Override
	public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
		this.beanFactory=beanFactory;
	}


}
