/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.annotations;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import org.boretti.drools.integration.drools5.DroolsServiceType;

/**
 * This annotation defines the interface or the class.
 * 
 * Usage of this annotation is mandatory:
 * <ul>
 * <li>On interface to mark it as DroolsInterface, that will support Drools stuff.</li>
 * <li>On classe to mark it as DroolsClass, that will be instrumented as compile time.</li>
 * </ul>
 * 
 * All attribute of the annotation can be ommited.
 * 
 * @author mbo
 *
 */
@Retention(RUNTIME)
@Target(TYPE)
public @interface DroolsService {
	/**
	 * This is the type of the service source.
	 * <ul>
	 * <li>COMPILED=Use a compiled drools rule</li>
	 * <li>SOURCE=Use a source drools file and compile it</li>
	 * <li>XML=This is a XML source drools file and compile it</li>
	 * </ul>
	 * @return
	 */
	DroolsServiceType type() default DroolsServiceType.COMPILED;
	
	/**
	 * This is the resource name.
	 * If not specified, use interface name.drl for source and interface name.cdrl for compiled one.
	 * @return
	 */
	String resourceName() default "";
}
